from enum import Enum


class FileType(str, Enum):
    """Enum to define a file type."""

    pdf = 'pdf'
    xml = 'xml'
    zip = 'zip'

class Steps(str,Enum):
    legal = "legal"
    logo = "logo"
    certificate = "certificate"


class InvoiceRelation(str, Enum):
    """Relation key in SAT catalogue for related invoices."""

    nota_de_credito = '01'
    nota_de_debito = '02'
    devolucion_de_mercancia = '03'
    sustitucion_de_cfdi_previos = '04'
    traslados_de_mercancia_facturados_previamente = '05'
    factura_por_traslados_previos = '06'
    aplicacion_de_anticipo = '07'
    pagos_en_parcialidades = '08'
    pagos_diferidos = '09'


class InvoiceType(str, Enum):
    """Type of an invoice."""

    ingreso = 'I'
    egreso = 'E'
    traslado = 'T'
    nomina = 'N'
    pago = 'P'


class InvoiceUse(str, Enum):
    """CFDI use code from SAT catalogue."""

    adquisicion_mercancias = 'G01'
    devoluciones_descuentos_bonificaciones = 'G02'
    gastos_en_general = 'G03'
    construcciones = 'I01'
    mobiliario_y_equipo_de_oficina = 'I02'
    equipo_de_transporte = 'I03'
    equipo_de_computo = 'I04'
    dados_troqueles_herramental = 'I05'
    comunicaciones_telefonicas = 'I06'
    comunicaciones_satelitales = 'I07'
    otra_maquinaria = 'I08'
    honorarios_medicos = 'D01'
    gastos_medicos_por_incapacidad = 'D02'
    gastos_funerales = 'D03'
    donativos = 'D04'
    intereses_por_creditos_hipotecarios = 'D05'
    aportaciones_voluntarias_sar = 'D06'
    prima_seguros_gastos_medicos = 'D07'
    gastos_transportacion_escolar = 'D08'
    cuentas_ahorro_pensiones = 'D09'
    servicios_educativos = 'D10'
    por_definir = 'P01'


class PaymentForm(str, Enum):
    """Payment from code according to SAT."""

    efectivo = '01'
    cheque_nominativo = '02'
    transferencia_electronica_de_fondos = '03'
    tarjeta_de_credito = '04'
    monedero_electronico = '05'
    dinero_electronico = '06'
    vales_de_despensa = '06'
    dacion_en_pago = '12'
    pago_por_subrogacion = '13'
    pago_por_consignacion = '14'
    condonacion = '15'
    compensacion = '17'
    novacion = '23'
    confusion = '24'
    remision_de_deuda = '25'
    prescripcion_o_caducidad = '26'
    a_satisfaccion_del_acreedor = '27'
    tarjeta_de_debito = '28'
    tarjeta_de_servicios = '29'
    aplicacion_de_anticipos = '30'
    intermediario_pagos = '31'
    por_definir = '99'


class PaymentMethod(str, Enum):
    """Payment method code according to SAT."""

    contado = 'PUE'
    parcialidades = 'PPD'



from typing import List, Optional
import datetime as dt

from pydantic import BaseModel

from .validators import sanitize_dict
from .enums import Steps


class CustomerAddress(BaseModel):
    """Address of a customer.

    Attributes:
        street (str): Street.
        exterior (str): Exterior place number.
        interior (str): Interior place number.
        neighborhood (str): 'Colonia'.
        city (str): City.
        municipality (str): 'Municipio or Alcaldía'.
        state (str): State of the address.
        zip (str): Postal code.
        country (str): Country.

    """

    street: Optional[str]
    exterior: Optional[str]
    interior: Optional[str]
    neighborhood: Optional[str]
    city: Optional[str]
    municipality: Optional[str]
    zip: Optional[str]
    state: Optional[str]
    country: Optional[str]

class Taxes(BaseModel):
    """Taxes basic info."""
    rate: float = 0.16
    type: str = "IVA"
    factor: Optional[str] = "Tasa"
    withholding : Optional[bool] = False

class LocalTaxes(BaseModel):
    """Local taxes info"""
    rate: float
    type: str
    withholding : Optional[bool] = False


class CustomerBasicInfo(BaseModel):
    """Customer's basic info"""

    id: str
    legal_name: str
    tax_id: str
    tax_system: str


class ItemPart(BaseModel):
    """Defines a part of an invoice item."""

    description: str
    product_key: str
    quantity: Optional[int] = 1
    sku: Optional[str]
    unit_price: Optional[float]
    customs_keys: Optional[List[str]]


class Namespace(BaseModel):
    """Namespace for spceial XML namespaces for an invoice."""

    prefix: Optional[str]
    uri: Optional[str]
    schema_location: Optional[str]


class ProductBasicInfo(BaseModel):
    """Product's basic info."""

    id: Optional[str]
    description: str
    product_key: str
    price: int
    tax_included: Optional[bool] = False
    taxability: Optional[str] = "01"
    taxes : Optional[List[Taxes]] = None
    local_taxes : Optional[LocalTaxes] = None
    unit_key: Optional[str]
    unit_name: Optional[str]

class InvoiceDocuments(BaseModel):
    relationship : str
    documents : List

class InvoiceStamp(BaseModel):
    signature : str
    date : dt.datetime
    sat_cert_number: str
    sat_signature: str

class PendingSteps(BaseModel):
    type: Optional[Steps]
    description : str

class Legal(BaseModel):
    name: str
    legal_name: str
    tax_system: Optional[str]
    website: Optional[str]
    phone: Optional[str]

class SanitizedDict(dict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        sanitize_dict(self)







"""

Product resource, it includes the class Resource and two request
classes to create and update the resource.

Author: Daniel Hernández - KEA

"""

import datetime as dt
from typing import ClassVar, Optional, cast, List

from pydantic import BaseModel
from pydantic.dataclasses import dataclass

from ..types.general import Taxes, LocalTaxes
from .base import Creatable, Queryable, Retrievable, Updatable, Deletable

class ProductRequest(BaseModel):

    """
    This request must be filled to `create` a Product.
    It contains all information necessary to create this resource.

    Attributes:
        description (str): Description of the product
        product_key (str): Key of the product
        price (float): Price unit of the product
        tax_included (bool): Flag if price includes taxes.
        taxability (str): Code that represents if the product is subject to taxes or not.
        taxes (List[Taxes]): List of taxes that should be applied to this product.
        local_taxes(List[LocalTaxes]): List of local taxes that should be applied to this product.
        unit_key (str): Unit of measure key
        unit_name (str): Represents the unit of measure of your product
        sku (str): Identifier for internal use designated by the company

    """

    description: str
    product_key: str 
    price: float
    tax_included: Optional[bool]
    taxability: Optional[str] = "01"
    taxes: Optional[List[Taxes]] = None
    local_taxes: Optional[List[LocalTaxes]]
    unit_key: Optional[str]
    unit_name: Optional[str]
    sku: Optional[str]

class ProductUpdateRequest(BaseModel):
    """
    This request must be filled to `update` a Product.
    It contains all information necessary to update this resource.

    Attributes:
        description (str): Description of the product
        product_key (str): Key of the product
        price (float): Price unit of the product
        tax_included (bool): Flag if price includes taxes.
        taxability (str): Code that represents if the product is subject to taxes or not.
        taxes (List[Taxes]): List of taxes that should be applied to this product.
        local_taxes(List[LocalTaxes]): List of local taxes that should be applied to this product.
        unit_key (str): Unit of measure key
        unit_name (str): Represents the unit of measure of your product
        sku (str): Identifier for internal use designated by the company

    """

    description: Optional[str]
    product_key: Optional[str]
    price: Optional[float]
    tax_included: Optional[bool]
    taxability: Optional[str] = "01"
    taxes: Optional[List[Taxes]] = None
    local_taxes: Optional[List[LocalTaxes]]
    unit_key: Optional[str]
    unit_name: Optional[str]
    sku: Optional[str]

@dataclass
class Product(Creatable, Queryable, Retrievable, Updatable, Deletable):

    _resource: ClassVar = 'products'

    created_at: dt.datetime
    livemode: bool
    description: str
    product_key: str 
    price: float
    tax_included: Optional[bool]
    taxability: Optional[str]
    taxes: Optional[List[Taxes]]
    local_taxes: Optional[List[LocalTaxes]]
    unit_key: Optional[str]
    unit_name: Optional[str]
    sku: Optional[str]

    @classmethod
    def create(cls, data: ProductRequest) -> 'Product':
        """Create a product.

        Args:
            data: All the request data to create a product.

        Returns:
            Product: The created product resource.

        """
        cleaned_data = data.dict(exclude_unset=True, exclude_none=True)
        return cast('Product', cls._create(**cleaned_data))

    @classmethod
    def update(cls, id: str, data: ProductUpdateRequest) -> 'Product':
        """Update a customer.

        Args:
            id: ID of the customer to be updated.
            data: Data to be updated.

        Returns:
            Product: The udpated product resource.

        """
        cleaned_data = data.dict(exclude_unset=True, exclude_none=True)
        return cast('Product', cls._update(id=id, **cleaned_data))